searchData={"items":[{"type":"module","title":"Unzip","doc":"Module to get files out of a zip. Works with local and remote files","ref":"Unzip.html"},{"type":"module","title":"Overview - Unzip","doc":"Unzip tries to solve problem of accessing files from a zip which is\nnot local (Aws S3, sftp etc). It does this by simply separating file\nsystem and zip implementation. Anything which implements\n`Unzip.FileAccess` can be used to get zip contents. Unzip relies on\nthe ability to seek and read of the file, This is due to the nature\nof zip file.  Files from the zip are read on demand.","ref":"Unzip.html#module-overview"},{"type":"module","title":"Usage - Unzip","doc":"# Unzip.LocalFile implements Unzip.FileAccess\n    zip_file = Unzip.LocalFile.open(\"foo/bar.zip\")\n\n    # `new` reads list of files by reading central directory found at the end of the zip\n    {:ok, unzip} = Unzip.new(zip_file)\n\n    # Alternatively if you have the zip file in memory as binary you can\n    # directly pass it to `Unzip.new(binary)` to unzip\n    #\n    # {:ok, unzip} = Unzip.new( >)\n\n    # returns list of files along with metadata\n    file_entries = Unzip.list_entries(unzip)\n\n    # returns decompressed file stream\n    stream = Unzip.file_stream!(unzip, \"baz.png\")\n\n\nSupports STORED and DEFLATE compression methods. Does not support zip64 specification yet","ref":"Unzip.html#module-usage"},{"type":"function","title":"Unzip.file_stream!/2","doc":"Returns decompressed file entry from the zip as a stream. `file_name` *must* be complete file path. File is read in the chunks of 65k","ref":"Unzip.html#file_stream!/2"},{"type":"function","title":"Unzip.list_entries/1","doc":"Returns list of files metadata. This does not make `pread` call as metadata is already by `new/1`.\n\nSee `Unzip.Entry` for metadata fields","ref":"Unzip.html#list_entries/1"},{"type":"function","title":"Unzip.new/1","doc":"Reads zip metadata from the passed zip file.\n\n`zip` must implement `Unzip.FileAccess` protocol.\n\nFetches the list of files present in the zip and other metadata by\nreading central directory found at the end of the zip.\n\nReturns `Unzip` struct which contains passed `zip` struct and zip\nmetadata.","ref":"Unzip.html#new/1"},{"type":"type","title":"Unzip.t/0","doc":"Struct holding zip related metadata, returned by the `new/1`.\n\nPublic fields:\n\n* `zip` - Zip struct passed to `new/1`\n\nRemaining fields are private and must not be accessed directly.","ref":"Unzip.html#t:t/0"},{"type":"module","title":"Unzip.Entry","doc":"File metadata returned by `Unzip.list_entries/1`\n\n  * `:file_name` - (string) File name with complete path. Directory files will have `/` at the end of their name\n\n  * `:last_modified_datetime` - (NaiveDateTime) last modified date and time of the file. It will be set to `nil` if datetime is invalid\n\n  * `:compressed_size` - (positive integer) Compressed file size in bytes\n\n  * `:uncompressed_size` - (positive integer) Uncompressed file size in bytes","ref":"Unzip.Entry.html"},{"type":"type","title":"Unzip.Entry.t/0","doc":"","ref":"Unzip.Entry.html#t:t/0"},{"type":"protocol","title":"Unzip.FileAccess","doc":"Should be implemented for the module which can read from the file system.","ref":"Unzip.FileAccess.html"},{"type":"function","title":"Unzip.FileAccess.pread/3","doc":"Reads the chunk of data from the `file`.\n\nIt should return `{:ok, binary}` where binary is the chunk found at `offset` with length `length`","ref":"Unzip.FileAccess.html#pread/3"},{"type":"function","title":"Unzip.FileAccess.size/1","doc":"Returns the size of the file in bytes.","ref":"Unzip.FileAccess.html#size/1"},{"type":"type","title":"Unzip.FileAccess.chunk/0","doc":"","ref":"Unzip.FileAccess.html#t:chunk/0"},{"type":"type","title":"Unzip.FileAccess.reason/0","doc":"","ref":"Unzip.FileAccess.html#t:reason/0"},{"type":"type","title":"Unzip.FileAccess.t/0","doc":"","ref":"Unzip.FileAccess.html#t:t/0"},{"type":"module","title":"Unzip.LocalFile","doc":"Simple module to access local zip file and example for implementing `Unzip.FileAccess`.\n\nNote: this is basic implementation, it is callers responsibility to handle resource properly","ref":"Unzip.LocalFile.html"},{"type":"function","title":"Unzip.LocalFile.close/1","doc":"","ref":"Unzip.LocalFile.html#close/1"},{"type":"function","title":"Unzip.LocalFile.open/1","doc":"","ref":"Unzip.LocalFile.html#open/1"},{"type":"exception","title":"Unzip.Error","doc":"","ref":"Unzip.Error.html"},{"type":"extras","title":"Unzip","doc":"# Unzip\n\n[![CI](https://github.com/akash-akya/unzip/actions/workflows/ci.yml/badge.svg)](https://github.com/akash-akya/unzip/actions/workflows/ci.yml)\n[![Hex.pm](https://img.shields.io/hexpm/v/unzip.svg)](https://hex.pm/packages/unzip)\n[![docs](https://img.shields.io/badge/docs-hexpm-blue.svg)](https://hexdocs.pm/unzip/)\n\nElixir library to stream zip file contents. Works with remote files. Supports Zip64.","ref":"readme.html"},{"type":"extras","title":"Overview - Unzip","doc":"Unzip tries to solve problem of unzipping files from different types of storage (Aws S3, SFTP server, in-memory etc). It separates type of storage from zip implementation. Unzip can stream zip contents from any type of storage which implements `Unzip.FileAccess` protocol. You can selectively stream files from zip without reading the complete zip. This saves bandwidth and decompression time if you only need few files from the zip. For example, if a zip file contains 100 files and we only want one file then `Unzip` access only that particular file","ref":"readme.html#overview"},{"type":"extras","title":"Installation - Unzip","doc":"```elixir\ndef deps do\n  [\n    {:unzip, \"~> x.x.x\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Unzip","doc":"```elixir\n# Unzip.LocalFile implements Unzip.FileAccess\nzip_file = Unzip.LocalFile.open(\"foo/bar.zip\")\n\n# `new` reads list of files by reading central directory found at the end of the zip\n{:ok, unzip} = Unzip.new(zip_file)\n\n# Alternatively if you have the zip file in memory as binary you can\n# directly pass it to `Unzip.new(binary)` to unzip\n#\n# {:ok, unzip} = Unzip.new( >)\n\n# presents already read files metadata\nfile_entries = Unzip.list_entries(unzip)\n\n# returns decompressed file stream\nstream = Unzip.file_stream!(unzip, \"baz.png\")\n```\n\nSupports STORED and DEFLATE compression methods. Supports zip64 specification.","ref":"readme.html#usage"},{"type":"extras","title":"Sample implementations of `Unzip.FileAccess` protocol - Unzip","doc":"For Aws S3 using [ExAws](https://hexdocs.pm/ex_aws/ExAws.html)\n\n```elixir\ndefmodule Unzip.S3File do\n  defstruct [:path, :bucket, :s3_config]\n  alias __MODULE__\n\n  def new(path, bucket, s3_config) do\n    %S3File{path: path, bucket: bucket, s3_config: s3_config}\n  end\nend\n\ndefimpl Unzip.FileAccess, for: Unzip.S3File do\n  alias ExAws.S3\n\n  def size(file) do\n    %{headers: headers} = S3.head_object(file.bucket, file.path) |> ExAws.request!(file.s3_config)\n\n    size =\n      headers\n      |> Enum.find(fn {k, _} -> String.downcase(k) == \"content-length\" end)\n      |> elem(1)\n      |> String.to_integer()\n\n    {:ok, size}\n  end\n\n  def pread(file, offset, length) do\n    {_, chunk} =\n      S3.Download.get_chunk(\n        %S3.Download{bucket: file.bucket, path: file.path, dest: nil},\n        %{start_byte: offset, end_byte: offset + length - 1},\n        file.s3_config\n      )\n\n    {:ok, chunk}\n  end\nend\n\n\n# Using S3File\n\naws_s3_config = ExAws.Config.new(:s3,\n  access_key_id: [\"key_id\", :instance_role],\n  secret_access_key: [\"key\", :instance_role]\n)\n\nfile = Unzip.S3File.new(\"pets.zip\", \"pics\", aws_s3_config)\n{:ok, unzip} = Unzip.new(file)\nfiles = Unzip.list_entries(unzip)\n\nUnzip.file_stream!(unzip, \"cats/kitty.png\")\n|> Stream.into(File.stream!(\"kitty.png\"))\n|> Stream.run()\n\n```\n\nFor zip file in SFTP server\n\n```elixir\ndefmodule Unzip.SftpFile do\n  defstruct [:channel_pid, :connection_ref, :handle, :file_path]\n  alias __MODULE__\n\n  def new(host, port, sftp_opts, file_path) do\n    :ok = :ssh.start()\n\n    {:ok, channel_pid, connection_ref} =\n      :ssh_sftp.start_channel(to_charlist(host), port, sftp_opts)\n\n    {:ok, handle} = :ssh_sftp.open(channel_pid, file_path, [:read, :raw, :binary])\n\n    %SftpFile{\n      channel_pid: channel_pid,\n      connection_ref: connection_ref,\n      handle: handle,\n      file_path: file_path\n    }\n  end\n\n  def close(file) do\n    :ssh_sftp.close(file.channel_pid, file.handle)\n    :ssh_sftp.stop_channel(file.channel_pid)\n    :ssh.close(file.connection_ref)\n    :ok\n  end\nend\n\ndefimpl Unzip.FileAccess, for: Unzip.SftpFile do\n  def size(file) do\n    {:ok, file_info} = :ssh_sftp.read_file_info(file.channel_pid, file.file_path)\n    {:ok, elem(file_info, 1)}\n  end\n\n  def pread(file, offset, length) do\n    :ssh_sftp.pread(file.channel_pid, file.handle, offset, length)\n  end\nend\n\n\n# Using SftpFile\n\nsftp_opts = [\n  user_interaction: false,\n  silently_accept_hosts: true,\n  rekey_limit: 1_000_000_000_000,\n  user: 'user',\n  password: 'password'\n]\n\nfile = Unzip.SftpFile.new('127.0.0.1', 22, sftp_opts, '/home/user/pics.zip')\n\ntry do\n  {:ok, unzip} = Unzip.new(file)\n  files = Unzip.list_entries(unzip)\n\n  Unzip.file_stream!(unzip, \"cats/kitty.png\")\n  |> Stream.into(File.stream!(\"kitty.png\"))\n  |> Stream.run()\nafter\n  Unzip.SftpFile.close(file)\nend\n\n```","ref":"readme.html#sample-implementations-of-unzip-fileaccess-protocol"}],"content_type":"text/markdown"}